# -*- coding: utf-8 -*-

"""Top-level package for pandas-log."""
from .pandas_log import *

__author__ = """Eyal Trabelsi"""
__email__ = "eyaltrabelsi@gmail.com"
__version__ = "0.0.0"

ORIGINAL_METHOD_PREFIX = "original_"
